package jaredbgreat.dldungeons.themes;


/*This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/	


import java.util.ArrayList;
import java.util.EnumSet;
import java.util.Random;

import net.minecraft.world.biome.BiomeGenBase;
import net.minecraftforge.common.BiomeDictionary.Type;

public class Theme {
	
	public String name;
	
	public static Theme classic;
	public static Theme nether;
	public static Theme end;
	
	public EnumSet<Type> biomes = EnumSet.noneOf(Type.class);
	//public static EnumSet<Type> notIn  = EnumSet.noneOf(Type.class);
	
	public int minY;
	public int maxY;
	public boolean buildFoundation;
	
	//TODO:  Biome restriction (especially important for the Nether)

	// Planning parameters
	public SizeElement sizes;
	// Not sure if I'll use all of them...	
	public Element outside;		// Roofless, wall-less rooms (intend for outdoors in surface dungeons)
	public Element liquids;		// Quantity of water / lava pools
	public Element subrooms;	// Rooms budding of from the main one
	public Element islands;		// Rooms inside rooms	
	public Element pillars;		// Uh, pillars / columns, duh!
	public Element fences;		// Uh, fences, duh!
	public Element symmetry;	// How symmetrical rooms are (technically, chance of axis mirroring)
	public Element variability;	// Inconsistency, that chance of using a different style in some place
	public Element degeneracy;	// Chance of walls / ceilings not spawning over air blocks (idea taken from Greymerk)
	public Element complexity;	// Basically how many shape primitives to add
	public Element verticle;	// How many height change and how much height change
	public Element entrances;	// Ways in and out / number of entrance/exit nodes (others are destination / treasure nodes)
	
	public int[] walls;
	public int[] floors;
	public int[] ceilings;
	public int[] fencing;
	public int[] liquid;
	public int[] pillarBlock;
	
	public ArrayList<String> commonMobs;
	public ArrayList<String> hardMobs;
	public ArrayList<String> bruteMobs;
	public ArrayList<String> eliteMobs;
	public ArrayList<String>[] allMobs;
	
	
	public Theme(int minY, int maxY, boolean buildFoundation,
				 SizeElement sizes, Element skies, Element liquids, 
				 Element subrooms, Element islands, Element pillars, Element fences, 
				 Element symmetry, Element variability, Element degeneracy, Element complexity, 
				 Element verticle, Element entrances, Element bigRooms,
				 int[] walls, int[] floors, int[] ceilings, int[] stairs,
				 int[] stairSlabs, int[] fencing, int[] liquid, int[] corner, 
				 int[] windowContent, String[] mobs, String[] hardMobs) {
		this.minY = minY;
		this.maxY = maxY;
		this.buildFoundation = buildFoundation;
		this.sizes = sizes;	
		this.outside = skies;
		this.liquids = liquids;
		this.subrooms = subrooms;
		this.islands = islands;	
		this.pillars = pillars;
		this.symmetry = symmetry;
		this.variability = variability;
		this.degeneracy = degeneracy;
		this.complexity = complexity;
		this.verticle = verticle;
		this.entrances = entrances;
		this.walls = walls;
		this.floors = floors;
		this.ceilings = ceilings;
		this.fencing = fencing;
		this.liquid = liquid;
		this.pillarBlock = corner;
		this.commonMobs = new ArrayList<String>(mobs.length);
		this.hardMobs = new ArrayList<String>(hardMobs.length);
		for(int i = 0; i < mobs.length; i++) {
			this.commonMobs.add(mobs[i]);
		} for(int i = 0; i < hardMobs.length; i++) {
			this.hardMobs.add(hardMobs[i]);
		}
		fixMobs();
	}
	
	
	public Theme() {
		minY = 10;
		maxY = 50;
		buildFoundation = false;
		sizes = new SizeElement(2, 5, 10, 5, 1);	
		outside = new Element(25,  0,  0,  0,  0,  0);
		liquids = new Element(1,  30, 50, 20, 10,  0);
		subrooms = new Element(0, 5, 25, 60, 25,  5);
		islands = new Element(5,  50, 10, 50, 20,  0);	
		pillars = new Element(5,  30, 60, 40, 20,  0);
		symmetry = new Element(5,  15, 30, 75, 25,  0);
		variability = new Element(5,  10, 25, 75, 50, 25);
		degeneracy = new Element(50,  5, 15, 50, 10,  0);
		complexity = new Element(5,  10, 25, 75, 15,  0);
		verticle = new Element(5,  10, 25, 20, 10,  0);
		entrances = new Element(2,   5, 25, 50, 15,  3);
		fences = new Element(15, 25, 55, 15,  0, 0);
		walls    = new int[]{98, 4, 24, 45};
		floors   = new int[]{98, 4, 3, 1}; 
		ceilings = new int[]{98, 4, 5, 43}; 
		fencing  = new int[]{139, 85}; 
		liquid   = new int[]{9, 11}; 
		pillarBlock = new int[]{98, 4, 24, 5, 43};
		String[] mobs = new String[]{"Zombie", "Skeleton", "Spider"};
		String[] hard = new String[]{"Creeper", "Enderman", "CaveSpider"};
		commonMobs = new ArrayList<String>(mobs.length);
		hardMobs = new ArrayList<String>(hard.length);
		bruteMobs = new ArrayList<String>(1);
		for(int i = 0; i < mobs.length; i++) {
			this.commonMobs.add(mobs[i]);
		} for(int i = 0; i < hard.length; i++) {
			this.hardMobs.add(hard[i]);
		}
		bruteMobs.add("Witch");
		fixMobs();
	}
	
	
	/*--------------------------------------*
	 * 			ADD THEMES                   *
	 ---------------------------------------*/
	
	
	public static void addDefaultTheme() {
		String[] mobs;
		String[] hard;
		
		classic = new Theme();
		
		nether = new Theme();
		nether.minY = 32;
		nether.maxY = 64;
		nether.buildFoundation = true;
		nether.walls = new int[]{112};
		nether.floors = new int[]{112};
		nether.ceilings = new int[]{112};
		nether.liquid = new int[]{11};
		nether.pillarBlock = new int[]{112, 49, 112, 49, 155};
		nether.liquids = new Element(0, 0, 5, 85, 10, 0);
		nether.verticle = new Element(0, 0, 5, 55, 15, 0);
		nether.outside = new Element(5, 10, 15, 5, 0, 0);
		nether.pillars = new Element(5, 15, 10, 5, 0, 0);
		nether.degeneracy = new Element(10, 0, 0, 0, 0, 0);
		nether.entrances = new Element(10, 0, 0, 0, 0, 0);
		mobs = new String[]{"Blaze", "LavaSlime", "PigZombie"};
		hard = new String[]{"Blaze", "Skeleton"};
		nether.commonMobs = new ArrayList<String>(mobs.length);
		nether.hardMobs = new ArrayList<String>(hard.length);
		for(int i = 0; i < mobs.length; i++) {
			nether.commonMobs.add(mobs[i]);
		} for(int i = 0; i < hard.length; i++) {
			nether.hardMobs.add(hard[i]);
		}
		nether.eliteMobs.add("WitherBoss");
		nether.fixMobs();
		
		end = new Theme();
		end.minY = 30;
		end.maxY = 60;
		end.walls = new int[]{121};
		end.floors = new int[]{121};
		end.ceilings = new int[]{121};
		end.liquid = new int[]{0};
		end.pillarBlock = new int[]{121, 49, 49, 49, 89};
		end.liquids = new Element(5, 0, 0, 0, 0, 0);
		end.verticle = new Element(0, 0, 5, 55, 15, 0);
		end.outside = new Element(5, 10, 15, 5, 0, 0);
		end.pillars = new Element(0, 5, 15, 10, 5, 0);
		end.degeneracy = new Element(10, 0, 0, 0, 0, 0);
		end.entrances = new Element(10, 0, 0, 0, 0, 0);
		end.commonMobs = new ArrayList<String>(mobs.length);
		end.hardMobs = new ArrayList<String>(hard.length);
		end.bruteMobs = new ArrayList<String>(hard.length);
		end.eliteMobs = new ArrayList<String>(hard.length);
		end.commonMobs.add("Enderman");
		end.hardMobs.add("Enderman");
		end.bruteMobs.add("Enderman");
		end.eliteMobs.add("WitherBoss");
		end.fixMobs();
	}
	
	
	public void fixMobs() {
		if(hardMobs == null) hardMobs = commonMobs;
		else if(hardMobs.isEmpty()) hardMobs = commonMobs;
		if(bruteMobs == null) bruteMobs = hardMobs;
		else if(bruteMobs.isEmpty()) bruteMobs = hardMobs;
		if(eliteMobs == null) eliteMobs = bruteMobs;
		else if(eliteMobs.isEmpty()) eliteMobs = bruteMobs;
		allMobs = new ArrayList[4];
		allMobs[0] = commonMobs;
		allMobs[1] = hardMobs;
		allMobs[2] = bruteMobs;
		allMobs[3] = eliteMobs;
	}
	
	
	public void biomeRegister() {
		BiomeLists.registerTheme(this);
//		BiomeLists.removeTheme(this);
	}
	
	
	@Override
	public String toString() {
		if(name == null) return "null";
		return name;
	}
	
}
